(function () {
  'use strict';

  angular.module('orchestra.common.components', [
      'orchestra.common.components.controllers',
      'orchestra.common.components.directives'
  ]);

  angular.module('orchestra.common.components.controllers', []);
  angular.module('orchestra.common.components.directives', []);
})();
