(function () {
  'use strict';

  angular.module('orchestra.common', [
    'orchestra.common.components',
    'orchestra.common.services'
  ]);

  angular.module('orchestra.common.services', []);
})();
