(function () {
  'use strict';

  angular.module('simple_workflow.v1.rate.module', [
    'simple_workflow.v1.rate.controllers',
    'simple_workflow.v1.rate.directives'
  ]);

  angular.module('simple_workflow.v1.rate.controllers', []);
  angular.module('simple_workflow.v1.rate.directives', []);

})();
