(function () {
  'use strict';

  angular.module('test_dir.v2.s2.module', [
    'test_dir.v2.s2.controllers',
    'test_dir.v2.s2.directives'
  ]);

  angular.module('test_dir.v2.s2.controllers', []);
  angular.module('test_dir.v2.s2.directives', []);

})();
