(function () {
  'use strict';

  angular.module('orchestra.project_management', [
      'orchestra.common.services',
      'orchestra.project_management.controllers',
      'orchestra.project_management.directives',
      'orchestra.project_management.services'
    ]);

  angular.module('orchestra.project_management.controllers', []);
  angular.module('orchestra.project_management.directives', []);
  angular.module('orchestra.project_management.services', []);
})();
